//CustomToolbar.cpp : Implementation of CustomToolbar

#include "stdafx.h"
#include "CustomToolbar.h"

#define EDITCTRL_CX		200
#define EDITCTRL_CY		22
#define BTN_CAPTION		_T("Search People")
#define IDM_DOSEARCH	(32001)

/////////////////////////////////////////////////////////////////////////////
// Constructor/Destructor

CCustomToolbar::CCustomToolbar() : m_pBrowser(NULL)
{
}

CCustomToolbar::~CCustomToolbar()
{
	SetBrowser(NULL);
	if (IsWindow())
	{
		DestroyWindow();
	}
}
/**
 *	WM_ handler 
 */
LRESULT CCustomToolbar::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int		iImageIndex = 0;
	int		iIndex = 0;
	RECT	rect = {0,0,0,0};
	DWORD	dwStyleEx = GetExtendedStyle();
	TBBUTTON Button;
	HIMAGELIST hImageList;

	// buttons with images and text
	dwStyleEx |= TBSTYLE_EX_MIXEDBUTTONS;
	SetExtendedStyle(dwStyleEx);

	// Sets the size of the TBBUTTON structure.
	//SetButtonStructSize(int nSize = sizeof(TBBUTTON))
	SetButtonStructSize();

	// add our button's caption to the toolbar window
	iIndex = AddStrings( BTN_CAPTION );

	// load our button's icon and create the image list to house it.
	HICON hIcon = LoadIcon( g_hInstance, MAKEINTRESOURCE(IDI_PEOPLE));
	hImageList = ImageList_Create(16,16, ILC_MASK|ILC_COLOR32, 1, 0);
	iImageIndex = ImageList_AddIcon(hImageList, hIcon);
	DestroyIcon(hIcon);
	// Set the toolbar's image
	SetImageList(hImageList);

	// add the button for the toolbar to the window
	ZeroMemory((void*)&Button, sizeof(TBBUTTON));
	Button.idCommand = IDM_DOSEARCH;
	Button.fsState = TBSTATE_ENABLED;
	Button.fsStyle = BTNS_BUTTON | BTNS_AUTOSIZE | BTNS_SHOWTEXT;
	Button.dwData = 0;
	Button.iString = iIndex;
	Button.iBitmap = 0;
	InsertButton(iIndex, &Button);

	// create our edit window and set the font.
	m_editWnd.Create(m_hWnd, rect, NULL, WS_CHILD|WS_VISIBLE, WS_EX_CLIENTEDGE);
	m_editWnd.SetFont(static_cast<HFONT>(GetStockObject(DEFAULT_GUI_FONT)));

	return 0;
}
/**
 *	WM_ handler 
 */
LRESULT CCustomToolbar::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// based on the size of the window area minus the size of the toolbar button, 
	// indent the toolbar so that we can place the edit box before the toolbar 
	// button. This will right justify the toolbar button in the toolbar and the 
	// edit box will use the vaction space to the left of the button but after the 
	// toolbar text as it's usable space.

	int		btnLen = 0;
	RECT	wndRect, btnRect;

	GetClientRect(&wndRect);
	GetItemRect( 0, &btnRect );
	btnLen = (btnRect.right - btnRect.left);
	// put a small spacing gap between the edit box's right edge and the toolbar button's left edge
	SetIndent(EDITCTRL_CX+3);
	// coordinates for edit control
	wndRect.left = 0;
	wndRect.right = EDITCTRL_CX;
	m_editWnd.MoveWindow(&wndRect, FALSE);

	return 0;
}
/**
 *	
 */
void CCustomToolbar::SetBrowser(IWebBrowser2* pBrowser)
{
	if ( m_pBrowser )	{
		m_pBrowser->Release();
	}
	m_pBrowser = pBrowser;
	if ( m_pBrowser )	{
		m_pBrowser->AddRef();
	}
}

/**
 *	WM_ handler 
 */
LRESULT CCustomToolbar::OnCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if ( !HIWORD(wParam) )
	{
		if ( LOWORD(wParam) == IDM_DOSEARCH )
		{
			doSearch();
			bHandled = FALSE;
			return 1;
		}
	}
	return 0;
}

/**
 *	WM_ handler 
 */
LRESULT CCustomToolbar::OnDoSearch(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	doSearch();
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////
// Internal Routines
/**
 *	usaully 
 */
void CCustomToolbar::SetFocusControl()
{
	m_editWnd.SetFocus();
}

/**
 *	this is an example of google call 	
 *  http://www.google.com/search?hl=en&q=Alan+Parker&btnG=Google+Search
 */
void CCustomToolbar::doSearch()
{
	int			count = 0;
	TCHAR		szText[MAX_PATH] = {0};
	VARIANT		vEmpty;
	_bstr_t		bsSite;

	// if we have a web browser pointer then try to navigate to the Google to do search.
	if (m_pBrowser)
	{
		VariantInit(&vEmpty);
		m_pBrowser->Stop();
		// if the user has entered stock quotes then append them to the url
		if ( m_editWnd.GetWindowTextLength() )
		{
			count = m_editWnd.GetWindowText( szText, MAX_PATH-1);
		}

		if ( count )
		{
			makeRequestString( szText, bsSite );
		}
		else 
		{
			bsSite = "http://www.google.com";
		}
		// have the webrowser navigate to the site URL requested depending on user input.
		m_pBrowser->Navigate(bsSite, &vEmpty, &vEmpty, &vEmpty, &vEmpty);
	}
}
/**
 *
 */
void CCustomToolbar::makeRequestString(LPCTSTR szIn, _bstr_t& bstrOut )
{
	TCHAR*	pGap = (TCHAR *) szIn;
	_bstr_t bstrTickers;
	
	while( (pGap=_tcschr(pGap,int(_T(' ')))) )
	{
		*pGap++ = _T('+');
	}

	bstrTickers = szIn;
	bstrOut = _T("http://www.google.com/search?h1=en&q=");
	bstrOut += bstrTickers;
	bstrOut += _T("&btnG=Google+Search");
}
