// ToolBand.cpp : Implementation of CToolBand

#include "stdafx.h"
#include "ToolBand.h"

#include <shlwapi.h>

#define	TOOLBAR_CX		250 
#define	TOOLBAR_CY		22
#define	TITLE_TOOLBAND  L"The Search Toolbar"

// CToolBand

CToolBand::CToolBand(): 
	m_bShow(FALSE), 
	m_dwBandId(0), 
	m_dwViewMode(0), 
	m_bEnterHelpMode(FALSE), 
	m_hWndRebar(NULL),
	m_pSite(NULL),
	m_pWebBrowser(NULL),
	m_pToolbarCtrl(NULL)
{
}

/**
 *
 */
BOOL CToolBand::RegisterAndCreateWindow()
{
	BOOL	rc = FALSE;
	RECT	rect;
	::GetClientRect(m_hWndRebar, &rect);
	m_reflectWnd.Create(m_hWndRebar, rect, NULL, WS_CHILD);
	m_pToolbarCtrl = m_reflectWnd.GetToolbarPtr();

	// The toolbar is the window that the host will be using so it is the window that is important.
	if ( m_pToolbarCtrl )
	{
		m_pToolbarCtrl->SetBrowser(m_pWebBrowser);
		m_pToolbarCtrl->GetEditPtr()->SetBand(this);
		rc = m_pToolbarCtrl->IsWindow();
	}
	return rc;
}

// IDeskBand
STDMETHODIMP CToolBand::GetBandInfo(DWORD dwBandID, DWORD dwViewMode, DESKBANDINFO* pdbi)
{
	m_dwBandId = dwBandID;
	m_dwViewMode = dwViewMode;

	if (pdbi)
	{
		if (pdbi->dwMask & DBIM_MINSIZE)
		{
			pdbi->ptMinSize.x = TOOLBAR_CX;
			pdbi->ptMinSize.y = TOOLBAR_CY;
		}
		if (pdbi->dwMask & DBIM_MAXSIZE)
		{
			pdbi->ptMaxSize.x = 0; // ignored
			pdbi->ptMaxSize.y = -1;	// width
		}
		if (pdbi->dwMask & DBIM_INTEGRAL)
		{
			pdbi->ptIntegral.x = 1; // ignored
			pdbi->ptIntegral.y = 1; // not sizeable
		}
		if (pdbi->dwMask & DBIM_ACTUAL)
		{
			pdbi->ptActual.x = TOOLBAR_CX;
			pdbi->ptActual.y = TOOLBAR_CY;
		}
		if (pdbi->dwMask & DBIM_TITLE)
		{
			wcscpy(pdbi->wszTitle, TITLE_TOOLBAND);
		}
		if (pdbi->dwMask & DBIM_BKCOLOR)
		{
			//Use the default background color by removing this flag.
			pdbi->dwMask &= ~DBIM_BKCOLOR;
		}
		if (pdbi->dwMask & DBIM_MODEFLAGS)
		{
            pdbi->dwModeFlags = DBIMF_NORMAL | DBIMF_VARIABLEHEIGHT | DBIMF_BREAK;
		}
	}
	return S_OK;
}

// IOleWindow
STDMETHODIMP CToolBand::GetWindow(HWND* phwnd)
{
	HRESULT hr = S_OK;
	if ( phwnd == NULL )
	{
		hr = E_INVALIDARG;
	}
	else
	{
		if ( m_pToolbarCtrl  )
			*phwnd = m_pToolbarCtrl->m_hWnd;
		else
			*phwnd = NULL;

	}
	return hr;
}

STDMETHODIMP CToolBand::ContextSensitiveHelp(BOOL fEnterMode)
{
	m_bEnterHelpMode = fEnterMode;
	return S_OK;
}

// IDockingWindow
STDMETHODIMP CToolBand::CloseDW(unsigned long dwReserved)
{
	return S_OK;
}

STDMETHODIMP CToolBand::ResizeBorderDW(const RECT* prcBorder, IUnknown* punkToolbarSite, BOOL fReserved)
{
	// Not used by any band object.
	return E_NOTIMPL;
}

STDMETHODIMP CToolBand::ShowDW(BOOL fShow)
{
	m_bShow = fShow;
	if ( m_pToolbarCtrl->IsWindow() )
	{
		m_pToolbarCtrl->ShowWindow( m_bShow? SW_SHOW : SW_HIDE );
	}
	return S_OK;
}
// IObjectWithSite
STDMETHODIMP CToolBand::SetSite(IUnknown* pUnkSite)
{
	//If a site is being held, release it.
	if ( m_pSite )
	{
		if ( m_pToolbarCtrl ) {
			m_pToolbarCtrl->SetBrowser( NULL );
		}

		m_pSite->Release();
		m_pSite = NULL;
	}

	//If punkSite is not NULL, a new site is being set.
	if ( pUnkSite )
	{
		//Get the parent window.
		IOleWindow  *pOleWindow = NULL;

		m_hWndRebar = NULL;

		if( SUCCEEDED(pUnkSite->QueryInterface(IID_IOleWindow, (LPVOID*)&pOleWindow)) )
		{
			pOleWindow->GetWindow(&m_hWndRebar);
			pOleWindow->Release();
		}

		if(!::IsWindow(m_hWndRebar))
			return E_FAIL;

		// Minimum of Explorer 4 required
        IServiceProviderPtr pServiceProvider = pUnkSite;
		if ( m_pWebBrowser )
		{
			m_pWebBrowser = NULL;
		}
		
        if ( FAILED(pServiceProvider->QueryService(SID_SWebBrowserApp, IID_IWebBrowser2, (void**)&m_pWebBrowser)) )
		{
            return E_FAIL;
		}

        if ( !RegisterAndCreateWindow() )
		{
			return E_FAIL;
		}

		//Get and keep the IInputObjectSite pointer.
		if ( SUCCEEDED(pUnkSite->QueryInterface(IID_IInputObjectSite, (LPVOID*)&m_pSite)) )
		{
			return S_OK;
		}

		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CToolBand::GetSite(REFIID riid, void **ppvSite)
{
	*ppvSite = NULL;

	if ( m_pSite )
	{
	   return m_pSite->QueryInterface(riid, ppvSite);
	}
	return E_FAIL;
}

// IPersist
STDMETHODIMP CToolBand::GetClassID(CLSID *pClassID)
{
	*pClassID = CLSID_CoToolBand;
	return S_OK;
}

// IPersistStream
STDMETHODIMP CToolBand::IsDirty(void)
{
	return S_FALSE;
}

STDMETHODIMP CToolBand::Load(IStream *pStm)
{
	return S_OK;
}

STDMETHODIMP CToolBand::Save(IStream *pStm, BOOL fClearDirty)
{
	return S_OK;
}

STDMETHODIMP CToolBand::GetSizeMax(ULARGE_INTEGER *pcbSize)
{
	return E_NOTIMPL;
}

// IInputObject
STDMETHODIMP CToolBand::UIActivateIO(BOOL fActivate, LPMSG pMsg)
{
    if ( fActivate && m_pToolbarCtrl )
		m_pToolbarCtrl->SetFocusControl();

	return S_OK;
}
/**
 *
 */
STDMETHODIMP CToolBand::HasFocusIO(void)
{
	HWND hWnd = ::GetFocus();
	// if any of the windows in our toolbar have focus then return S_OK else S_FALSE.
	if ( m_pToolbarCtrl )
	{
		if ( m_pToolbarCtrl->m_hWnd == hWnd ||
			 m_pToolbarCtrl->GetEditPtr()->m_hWnd == hWnd )
		{
			return S_OK;
		}
	}

    return S_FALSE;
}

/**
 *
 */
STDMETHODIMP CToolBand::TranslateAcceleratorIO(LPMSG pMsg)
{
	if ( m_pToolbarCtrl )
		return m_pToolbarCtrl->GetEditPtr()->TranslateAcceleratorIO(pMsg);
	else
		return TranslateAcceleratorIO(pMsg);
}
/**
 *
 */
void CToolBand::FocusChange( BOOL bHaveFocus)
{
	if ( m_pSite )
	{
		IUnknown* pUnk = NULL;
		if (SUCCEEDED(QueryInterface(IID_IUnknown, (LPVOID*)&pUnk)) && pUnk != NULL)
		{
			m_pSite->OnFocusChangeIS(pUnk, bHaveFocus);
			pUnk->Release();
			pUnk = NULL;
		}
	}
}
